

set -ex



pip check
pip list
pip list | grep -iE 'hatch-nodejs-version\s*0\.3\.1'
pytest -vvv --capture=tee-sys --cov=hatch_nodejs_version --cov-report=term-missing:skip-covered --no-cov-on-fail --cov-fail-under=80
exit 0
